@@ -31,8 +31,9 @@
 #include "cintltst.h"
 #include "ubrkimpl.h"
 #include "toolutil.h" /* for uprv_fileExists() */
-#include <stdlib.h>
+#include <stdbool.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 /* includes for TestSwapData() */
 #include "udataswp.h"
@@ -570,7 +571,7 @@ isAcceptable1(void *context,
         pInfo->formatVersion[0]==3 )
     {
         log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable1()\n", name, type);
-        return TRUE;
+        return true;
     } else {
         log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable1():-\n"
             "\tsize              = %d\n"
@@ -583,7 +584,7 @@ isAcceptable1(void *context,
             pInfo->dataVersion[0], pInfo->dataFormat[0], pInfo->dataFormat[1], pInfo->dataFormat[2], 
             pInfo->dataFormat[3]);  
         log_verbose("Call another verifying function to accept the data\n");
-        return FALSE;
+        return false;
     }
 }
 
@@ -607,11 +608,11 @@ isAcceptable2(void *context,
         pInfo->dataVersion[0]==unicodeVersion[0] )
     {
         log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable2()\n", name, type);
-        return TRUE;
+        return true;
     } else {
         log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable2()\n", name, type);
 
-        return FALSE;
+        return false;
     }
 
 
@@ -633,10 +634,10 @@ isAcceptable3(void *context,
         pInfo->dataVersion[0]==1   ) {
         log_verbose("The data from \"%s.%s\" IS acceptable using the verifying function isAcceptable3()\n", name, type);
 
-        return TRUE;
+        return true;
     } else {
         log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable3()\n", name, type);
-        return FALSE;
+        return false;
     }
 
 
@@ -733,10 +734,10 @@ isAcceptable(void *context,
         *((int*)context) == 2 ) {
         log_verbose("The data from\"%s.%s\" IS acceptable using the verifying function isAcceptable()\n", name, type);
 
-        return TRUE;
+        return true;
     } else {
         log_verbose("The data from \"%s.%s\" IS NOT acceptable using the verifying function isAcceptable()\n", name, type);
-        return FALSE;
+        return false;
     }
 }
 
@@ -1156,7 +1157,7 @@ static void TestICUDataName()
     }
 
     /* Only major number is needed. */
-    sprintf(expectDataName, "%s%d%c",
+    snprintf(expectDataName, sizeof(expectDataName), "%s%d%c",
                 "icudt",
                 (int)icuVersion[0],
                 typeChar);
@@ -1625,7 +1626,7 @@ TestSwapCase(UDataMemory *pData, const char *name,
 
 static void U_CALLCONV
 printErrorToString(void *context, const char *fmt, va_list args) {
-    vsprintf((char *)context, fmt, args);
+    vsnprintf((char *)context, 100, fmt, args);
 }
 
 #if !UCONFIG_NO_FILE_IO && !UCONFIG_NO_LEGACY_CONVERSION
