@@ -41,7 +41,7 @@ public:
     StringCaseTest();
     virtual ~StringCaseTest();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=0);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=0) override;
 
     void TestCaseConversion();
 
@@ -51,8 +51,10 @@ public:
                         void *iter, const char *localeID, uint32_t options);
     void TestCasing();
     void TestTitleOptions();
+    void TestDutchTitle();
     void TestFullCaseFoldingIterator();
     void TestGreekUpper();
+    void TestArmenian();
     void TestLongUpper();
     void TestMalformedUTF8();
     void TestBufferOverflow();
@@ -94,9 +96,11 @@ StringCaseTest::runIndexedTest(int32_t index, UBool exec, const char *&name, cha
 #if !UCONFIG_NO_BREAK_ITERATION && !UCONFIG_NO_FILE_IO && !UCONFIG_NO_LEGACY_CONVERSION
     TESTCASE_AUTO(TestCasing);
     TESTCASE_AUTO(TestTitleOptions);
+    TESTCASE_AUTO(TestDutchTitle);
 #endif
     TESTCASE_AUTO(TestFullCaseFoldingIterator);
     TESTCASE_AUTO(TestGreekUpper);
+    TESTCASE_AUTO(TestArmenian);
     TESTCASE_AUTO(TestLongUpper);
     TESTCASE_AUTO(TestMalformedUTF8);
     TESTCASE_AUTO(TestBufferOverflow);
@@ -121,21 +125,21 @@ StringCaseTest::runIndexedTest(int32_t index, UBool exec, const char *&name, cha
 void
 StringCaseTest::TestCaseConversion()
 {
-    static const UChar uppercaseGreek[] =
+    static const char16_t uppercaseGreek[] =
         { 0x399, 0x395, 0x3a3, 0x3a5, 0x3a3, 0x20, 0x03a7, 0x3a1, 0x399, 0x3a3, 0x3a4,
         0x39f, 0x3a3, 0 };
         // "IESUS CHRISTOS"
 
-    static const UChar lowercaseGreek[] = 
+    static const char16_t lowercaseGreek[] =
         { 0x3b9, 0x3b5, 0x3c3, 0x3c5, 0x3c2, 0x20, 0x03c7, 0x3c1, 0x3b9, 0x3c3, 0x3c4,
         0x3bf, 0x3c2, 0 };
         // "iesus christos"
 
-    static const UChar lowercaseTurkish[] = 
+    static const char16_t lowercaseTurkish[] =
         { 0x69, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x2c, 0x20, 0x6e, 0x6f, 0x74, 0x20, 0x63, 0x6f, 
         0x6e, 0x73, 0x74, 0x61, 0x6e, 0x74, 0x0131, 0x6e, 0x6f, 0x70, 0x6c, 0x65, 0x21, 0 };
 
-    static const UChar uppercaseTurkish[] = 
+    static const char16_t uppercaseTurkish[] =
         { 0x54, 0x4f, 0x50, 0x4b, 0x41, 0x50, 0x49, 0x20, 0x50, 0x41, 0x4c, 0x41, 0x43, 0x45, 0x2c, 0x20,
         0x0130, 0x53, 0x54, 0x41, 0x4e, 0x42, 0x55, 0x4c, 0 };
     
@@ -196,7 +200,7 @@ StringCaseTest::TestCaseConversion()
 
     // more string case mapping tests with the new implementation
     {
-        static const UChar
+        static const char16_t
 
         beforeLower[]= { 0x61, 0x42, 0x49,  0x3a3, 0xdf, 0x3a3, 0x2f, 0xd93f, 0xdfff },
         lowerRoot[]=   { 0x61, 0x62, 0x69,  0x3c3, 0xdf, 0x3c2, 0x2f, 0xd93f, 0xdfff },
@@ -212,48 +216,48 @@ StringCaseTest::TestCaseConversion()
         UnicodeString s;
 
         /* lowercase with root locale */
-        s=UnicodeString(FALSE, beforeLower, UPRV_LENGTHOF(beforeLower));
+        s=UnicodeString(false, beforeLower, UPRV_LENGTHOF(beforeLower));
         s.toLower("");
         if( s.length()!=UPRV_LENGTHOF(lowerRoot) ||
-            s!=UnicodeString(FALSE, lowerRoot, s.length())
+            s!=UnicodeString(false, lowerRoot, s.length())
         ) {
-            errln("error in toLower(root locale)=\"" + s + "\" expected \"" + UnicodeString(FALSE, lowerRoot, UPRV_LENGTHOF(lowerRoot)) + "\"");
+            errln("error in toLower(root locale)=\"" + s + "\" expected \"" + UnicodeString(false, lowerRoot, UPRV_LENGTHOF(lowerRoot)) + "\"");
         }
 
         /* lowercase with turkish locale */
-        s=UnicodeString(FALSE, beforeLower, UPRV_LENGTHOF(beforeLower));
+        s=UnicodeString(false, beforeLower, UPRV_LENGTHOF(beforeLower));
         s.setCharAt(0, beforeLower[0]).toLower(Locale("tr"));
         if( s.length()!=UPRV_LENGTHOF(lowerTurkish) ||
-            s!=UnicodeString(FALSE, lowerTurkish, s.length())
+            s!=UnicodeString(false, lowerTurkish, s.length())
         ) {
-            errln("error in toLower(turkish locale)=\"" + s + "\" expected \"" + UnicodeString(FALSE, lowerTurkish, UPRV_LENGTHOF(lowerTurkish)) + "\"");
+            errln("error in toLower(turkish locale)=\"" + s + "\" expected \"" + UnicodeString(false, lowerTurkish, UPRV_LENGTHOF(lowerTurkish)) + "\"");
         }
 
         /* uppercase with root locale */
-        s=UnicodeString(FALSE, beforeUpper, UPRV_LENGTHOF(beforeUpper));
+        s=UnicodeString(false, beforeUpper, UPRV_LENGTHOF(beforeUpper));
         s.setCharAt(0, beforeUpper[0]).toUpper(Locale(""));
         if( s.length()!=UPRV_LENGTHOF(upperRoot) ||
-            s!=UnicodeString(FALSE, upperRoot, s.length())
+            s!=UnicodeString(false, upperRoot, s.length())
         ) {
-            errln("error in toUpper(root locale)=\"" + s + "\" expected \"" + UnicodeString(FALSE, upperRoot, UPRV_LENGTHOF(upperRoot)) + "\"");
+            errln("error in toUpper(root locale)=\"" + s + "\" expected \"" + UnicodeString(false, upperRoot, UPRV_LENGTHOF(upperRoot)) + "\"");
         }
 
         /* uppercase with turkish locale */
-        s=UnicodeString(FALSE, beforeUpper, UPRV_LENGTHOF(beforeUpper));
+        s=UnicodeString(false, beforeUpper, UPRV_LENGTHOF(beforeUpper));
         s.toUpper(Locale("tr"));
         if( s.length()!=UPRV_LENGTHOF(upperTurkish) ||
-            s!=UnicodeString(FALSE, upperTurkish, s.length())
+            s!=UnicodeString(false, upperTurkish, s.length())
         ) {
-            errln("error in toUpper(turkish locale)=\"" + s + "\" expected \"" + UnicodeString(FALSE, upperTurkish, UPRV_LENGTHOF(upperTurkish)) + "\"");
+            errln("error in toUpper(turkish locale)=\"" + s + "\" expected \"" + UnicodeString(false, upperTurkish, UPRV_LENGTHOF(upperTurkish)) + "\"");
         }
 
         /* uppercase a short string with root locale */
-        s=UnicodeString(FALSE, beforeMiniUpper, UPRV_LENGTHOF(beforeMiniUpper));
+        s=UnicodeString(false, beforeMiniUpper, UPRV_LENGTHOF(beforeMiniUpper));
         s.setCharAt(0, beforeMiniUpper[0]).toUpper("");
         if( s.length()!=UPRV_LENGTHOF(miniUpper) ||
-            s!=UnicodeString(FALSE, miniUpper, s.length())
+            s!=UnicodeString(false, miniUpper, s.length())
         ) {
-            errln("error in toUpper(root locale)=\"" + s + "\" expected \"" + UnicodeString(FALSE, miniUpper, UPRV_LENGTHOF(miniUpper)) + "\"");
+            errln("error in toUpper(root locale)=\"" + s + "\" expected \"" + UnicodeString(false, miniUpper, UPRV_LENGTHOF(miniUpper)) + "\"");
         }
     }
 
@@ -436,7 +440,7 @@ StringCaseTest::TestCasingImpl(const UnicodeString &input,
 #if !UCONFIG_NO_BREAK_ITERATION
     case TEST_TITLE:
         name="toTitle";
-        result.toTitle((BreakIterator *)iter, locale, options);
+        result.toTitle(static_cast<BreakIterator *>(iter), locale, options);
         break;
 #endif
     case TEST_FOLD:
@@ -449,11 +453,12 @@ StringCaseTest::TestCasingImpl(const UnicodeString &input,
     }
     if(result!=output) {
         dataerrln("error: UnicodeString.%s() got a wrong result for a test case from casing.res", name);
+        dataerrln(UnicodeString("input = [") + input + "], expected = [" + output + "], actual = [" + result + "]");
     }
 #if !UCONFIG_NO_BREAK_ITERATION
     if(whichCase==TEST_TITLE && options==0) {
         result=input;
-        result.toTitle((BreakIterator *)iter, locale);
+        result.toTitle(static_cast<BreakIterator *>(iter), locale);
         if(result!=output) {
             dataerrln("error: UnicodeString.toTitle(options=0) got a wrong result for a test case from casing.res");
         }
@@ -463,14 +468,14 @@ StringCaseTest::TestCasingImpl(const UnicodeString &input,
     // UTF-8
     char utf8In[100], utf8Out[100];
     int32_t utf8InLength, utf8OutLength, resultLength;
-    UChar *buffer;
+    char16_t *buffer;
 
     IcuTestErrorCode errorCode(*this, "TestCasingImpl");
     LocalUCaseMapPointer csm(ucasemap_open(localeID, options, errorCode));
 #if !UCONFIG_NO_BREAK_ITERATION
-    if(iter!=NULL) {
+    if(iter!=nullptr) {
         // Clone the break iterator so that the UCaseMap can safely adopt it.
-        UBreakIterator *clone=ubrk_safeClone((UBreakIterator *)iter, NULL, NULL, errorCode);
+        UBreakIterator *clone=ubrk_safeClone(static_cast<UBreakIterator *>(iter), nullptr, nullptr, errorCode);
         ucasemap_setBreakIterator(csm.getAlias(), clone, errorCode);
     }
 #endif
@@ -544,7 +549,7 @@ StringCaseTest::TestCasing() {
                 errln("TestCasing failed to createTestData(%s) - %s", dataNames[whichCase], u_errorName(status));
                 break;
             }
-            const DataMap *myCase = NULL;
+            const DataMap *myCase = nullptr;
             while(casingTest->nextCase(myCase, status)) {
                 input = myCase->getString("Input", status);
                 output = myCase->getString("Output", status);
@@ -558,26 +563,26 @@ StringCaseTest::TestCasing() {
                 if(whichCase==TEST_TITLE) {
                     type = myCase->getInt("Type", status);
                     if(type>=0) {
-                        iter.adoptInstead(ubrk_open((UBreakIteratorType)type, cLocaleID, NULL, 0, &status));
+                        iter.adoptInstead(ubrk_open((UBreakIteratorType)type, cLocaleID, nullptr, 0, &status));
                     } else if(type==-2) {
                         // Open a trivial break iterator that only delivers { 0, length }
                         // or even just { 0 } as boundaries.
-                        static const UChar rules[] = { 0x2e, 0x2a, 0x3b };  // ".*;"
+                        static const char16_t rules[] = { 0x2e, 0x2a, 0x3b };  // ".*;"
                         UParseError parseError;
-                        iter.adoptInstead(ubrk_openRules(rules, UPRV_LENGTHOF(rules), NULL, 0, &parseError, &status));
+                        iter.adoptInstead(ubrk_openRules(rules, UPRV_LENGTHOF(rules), nullptr, 0, &parseError, &status));
                     }
                 }
 #endif
                 options = 0;
                 if(whichCase==TEST_TITLE || whichCase==TEST_FOLD) {
                     optionsString = myCase->getString("Options", status);
-                    if(optionsString.indexOf((UChar)0x54)>=0) {  // T
+                    if(optionsString.indexOf((char16_t)0x54)>=0) {  // T
                         options|=U_FOLD_CASE_EXCLUDE_SPECIAL_I;
                     }
-                    if(optionsString.indexOf((UChar)0x4c)>=0) {  // L
+                    if(optionsString.indexOf((char16_t)0x4c)>=0) {  // L
                         options|=U_TITLECASE_NO_LOWERCASE;
                     }
-                    if(optionsString.indexOf((UChar)0x41)>=0) {  // A
+                    if(optionsString.indexOf((char16_t)0x41)>=0) {  // A
                         options|=U_TITLECASE_NO_BREAK_ADJUSTMENT;
                     }
                 }
@@ -593,7 +598,7 @@ StringCaseTest::TestCasing() {
                 }
 
 #if !UCONFIG_NO_BREAK_ITERATION
-                iter.adoptInstead(NULL);
+                iter.adoptInstead(nullptr);
 #endif
             }
         }
@@ -603,9 +608,9 @@ StringCaseTest::TestCasing() {
     // more tests for API coverage
     status=U_ZERO_ERROR;
     input=UNICODE_STRING_SIMPLE("sTrA\\u00dfE").unescape();
-    (result=input).toTitle(NULL);
+    (result=input).toTitle(nullptr);
     if(result!=UNICODE_STRING_SIMPLE("Stra\\u00dfe").unescape()) {
-        dataerrln("UnicodeString::toTitle(NULL) failed.");
+        dataerrln("UnicodeString::toTitle(nullptr) failed.");
     }
 #endif
 }
@@ -665,6 +670,106 @@ StringCaseTest::TestTitleOptions() {
 #endif
 }
 
+#if !UCONFIG_NO_BREAK_ITERATION
+void StringCaseTest::TestDutchTitle() {
+    IcuTestErrorCode errorCode(*this, "TestDutchTitle");
+
+    Locale nl("nl");  // Dutch
+    LocalPointer<BreakIterator> iter(
+        BreakIterator::createWordInstance(nl, errorCode));
+    
+    // Dutch titlecase check in English
+    TestCasingImpl(
+        u"ijssel igloo IJMUIDEN",
+        u"Ijssel Igloo Ijmuiden",
+        TEST_TITLE,
+        nullptr, 
+        "en",
+        0);
+
+    // Dutch titlecase check in Dutch
+    TestCasingImpl(
+        u"ijssel igloo IJMUIDEN", 
+        u"IJssel Igloo IJmuiden", 
+        TEST_TITLE,
+        nullptr, 
+        "nl",
+        0);
+
+    // Dutch titlecase check in Dutch with nolowercase option
+    if (U_SUCCESS(errorCode)) {
+        iter->setText(u"ijssel igloo IjMUIdEN iPoD ijenough");
+        TestCasingImpl(
+            u"ijssel igloo IjMUIdEN iPoD ijenough", 
+            u"IJssel Igloo IJMUIdEN IPoD IJenough", 
+            TEST_TITLE,
+            nullptr, 
+            "nl",
+            U_TITLECASE_NO_LOWERCASE);
+    }
+
+    errorCode.reset();
+
+    // Accented IJ testing
+
+    struct dutchTitleTestCase {
+        const UnicodeString input;
+        const UnicodeString expectedFull;
+        const UnicodeString expectedOnlyChanged;
+    } dutchTitleTestCases[] = {
+        // input,            expectedFull,      expectedOnlyChanged
+        {u"ij",              u"IJ",             u"IJ"},
+        {u"IJ",              u"IJ",             u""},
+        {u"íj́",              u"ÍJ́",             u"ÍJ"},
+        {u"ÍJ́",              u"ÍJ́",             u""},
+        {u"íJ́",              u"ÍJ́",             u"Í"},
+        {u"Ij́",              u"Ij́",             u""},
+        {u"ij́",              u"Ij́",             u"I"},
+        {u"ïj́",              u"Ïj́",             u"Ï"},
+        {u"íj\u0308",        u"Íj\u0308",       u"Í"},
+        {u"íj́\U0001D16E",    u"Íj́\U0001D16E",   u"Í"},
+        {u"íj\u1ABE",        u"Íj\u1ABE",       u"Í"},
+
+        {u"ijabc",              u"IJabc",             u"IJ"},
+        {u"IJabc",              u"IJabc",             u""},
+        {u"íj́abc",              u"ÍJ́abc",             u"ÍJ"},
+        {u"ÍJ́abc",              u"ÍJ́abc",             u""},
+        {u"íJ́abc",              u"ÍJ́abc",             u"Í"},
+        {u"Ij́abc",              u"Ij́abc",             u""},
+        {u"ij́abc",              u"Ij́abc",             u"I"},
+        {u"ïj́abc",              u"Ïj́abc",             u"Ï"},
+        {u"íjabc\u0308",        u"Íjabc\u0308",       u"Í"},
+        {u"íj́abc\U0001D16E",    u"ÍJ́abc\U0001D16E",   u"ÍJ"},
+        {u"íjabc\u1ABE",        u"Íjabc\u1ABE",       u"Í"},
+
+        // Bug ICU-21919
+        {u"Í",                  u"Í",                 u""},
+    };
+
+    for (const auto& cas : dutchTitleTestCases) {
+        const UnicodeString &input = cas.input;
+        const UnicodeString &expectedFull = cas.expectedFull;
+        const UnicodeString &expectedOnlyChanged = cas.expectedOnlyChanged;
+
+        for (const auto& isOnlyChanged : {true, false}) {
+            uint32_t testOptions = U_TITLECASE_NO_LOWERCASE
+                | (isOnlyChanged ? U_OMIT_UNCHANGED_TEXT : 0);
+            
+            const UnicodeString &expected = isOnlyChanged ? expectedOnlyChanged : expectedFull;
+            
+            TestCasingImpl(
+                input,
+                expected,
+                TEST_TITLE,
+                nullptr,
+                "nl",
+                testOptions
+            );
+        }
+    }
+}
+#endif
+
 void
 StringCaseTest::TestFullCaseFoldingIterator() {
     UnicodeString ffi=UNICODE_STRING_SIMPLE("ffi");
@@ -718,7 +823,7 @@ StringCaseTest::assertGreekUpper(const char16_t *s, const char16_t *expected) {
     };
     for (int32_t i = 0; i < UPRV_LENGTHOF(capacities); ++i) {
         int32_t cap = capacities[i];
-        UChar *dest16 = result16.getBuffer(expected16.length() + 1);
+        char16_t *dest16 = result16.getBuffer(expected16.length() + 1);
         u_memset(dest16, 0x55AA, result16.getCapacity());
         UErrorCode errorCode = U_ZERO_ERROR;
         length = u_strToUpper(dest16, cap, s16.getBuffer(), s16.length(), "el", &errorCode);
@@ -786,7 +891,7 @@ StringCaseTest::assertGreekUpper(const char16_t *s, const char16_t *expected) {
 
 void
 StringCaseTest::TestGreekUpper() {
-    // http://bugs.icu-project.org/trac/ticket/5456
+    // https://unicode-org.atlassian.net/browse/ICU-5456
     assertGreekUpper(u"άδικος, κείμενο, ίριδα", u"ΑΔΙΚΟΣ, ΚΕΙΜΕΝΟ, ΙΡΙΔΑ");
     // https://bugzilla.mozilla.org/show_bug.cgi?id=307039
     // https://bug307039.bmoattachments.org/attachment.cgi?id=194893
@@ -814,6 +919,26 @@ StringCaseTest::TestGreekUpper() {
     assertGreekUpper(u"ή.", u"Ή.");
 }
 
+void StringCaseTest::TestArmenian() {
+    Locale hy("hy");  // Eastern Armenian
+    Locale hyw("hyw");  // Western Armenian
+    Locale root = Locale::getRoot();
+    // See ICU-13416:
+    // և ligature ech-yiwn
+    // uppercases to ԵՒ=ech+yiwn by default and in Western Armenian,
+    // but to ԵՎ=ech+vew in Eastern Armenian.
+    UnicodeString s(u"և Երևանի");
+
+    assertEquals("upper root", u"ԵՒ ԵՐԵՒԱՆԻ", UnicodeString(s).toUpper(root));
+    assertEquals("upper hy", u"ԵՎ ԵՐԵՎԱՆԻ", UnicodeString(s).toUpper(hy));
+    assertEquals("upper hyw", u"ԵՒ ԵՐԵՒԱՆԻ", UnicodeString(s).toUpper(hyw));
+#if !UCONFIG_NO_BREAK_ITERATION
+    assertEquals("title root", u"Եւ Երևանի", UnicodeString(s).toTitle(nullptr, root));
+    assertEquals("title hy", u"Եվ Երևանի", UnicodeString(s).toTitle(nullptr, hy));
+    assertEquals("title hyw", u"Եւ Երևանի", UnicodeString(s).toTitle(nullptr, hyw));
+#endif
+}
+
 void
 StringCaseTest::TestLongUpper() {
     if (quick) {
@@ -826,8 +951,8 @@ StringCaseTest::TestLongUpper() {
     int32_t length = 0x40000004;  // more than 1G UChars
     UnicodeString s(length, (UChar32)0x390, length);
     UnicodeString result;
-    UChar *dest = result.getBuffer(length + 1);
-    if (s.isBogus() || dest == NULL) {
+    char16_t *dest = result.getBuffer(length + 1);
+    if (s.isBogus() || dest == nullptr) {
         logln("Out of memory, unable to run this test on this machine.");
         return;
     }
@@ -898,7 +1023,7 @@ void StringCaseTest::TestBufferOverflow() {
     UnicodeString data("hello world");
     int32_t result;
 #if !UCONFIG_NO_BREAK_ITERATION
-    result = ucasemap_toTitle(csm.getAlias(), NULL, 0, data.getBuffer(), data.length(), errorCode);
+    result = ucasemap_toTitle(csm.getAlias(), nullptr, 0, data.getBuffer(), data.length(), errorCode);
     if (errorCode.get() != U_BUFFER_OVERFLOW_ERROR || result != data.length()) {
         errln("%s:%d ucasemap_toTitle(\"hello world\") failed: "
               "expected (U_BUFFER_OVERFLOW_ERROR, %d), got (%s, %d)",
@@ -910,7 +1035,7 @@ void StringCaseTest::TestBufferOverflow() {
     std::string data_utf8;
     data.toUTF8String(data_utf8);
 #if !UCONFIG_NO_BREAK_ITERATION
-    result = ucasemap_utf8ToTitle(csm.getAlias(), NULL, 0, data_utf8.c_str(), static_cast<int32_t>(data_utf8.length()), errorCode);
+    result = ucasemap_utf8ToTitle(csm.getAlias(), nullptr, 0, data_utf8.c_str(), static_cast<int32_t>(data_utf8.length()), errorCode);
     if (errorCode.get() != U_BUFFER_OVERFLOW_ERROR || result != (int32_t)data_utf8.length()) {
         errln("%s:%d ucasemap_toTitle(\"hello world\") failed: "
               "expected (U_BUFFER_OVERFLOW_ERROR, %d), got (%s, %d)",
@@ -948,32 +1073,32 @@ void StringCaseTest::TestEdits() {
     assertFalse("edits done: copyErrorTo", edits.copyErrorTo(outErrorCode));
 
     static const EditChange coarseExpectedChanges[] = {
-            { FALSE, 10003, 10003 },
-            { TRUE, 103106, 104013 }
+            { false, 10003, 10003 },
+            { true, 103106, 104013 }
     };
     TestUtility::checkEditsIter(*this, u"coarse",
             edits.getCoarseIterator(), edits.getCoarseIterator(),
-            coarseExpectedChanges, UPRV_LENGTHOF(coarseExpectedChanges), TRUE, errorCode);
+            coarseExpectedChanges, UPRV_LENGTHOF(coarseExpectedChanges), true, errorCode);
     TestUtility::checkEditsIter(*this, u"coarse changes",
             edits.getCoarseChangesIterator(), edits.getCoarseChangesIterator(),
-            coarseExpectedChanges, UPRV_LENGTHOF(coarseExpectedChanges), FALSE, errorCode);
+            coarseExpectedChanges, UPRV_LENGTHOF(coarseExpectedChanges), false, errorCode);
 
     static const EditChange fineExpectedChanges[] = {
-            { FALSE, 10003, 10003 },
-            { TRUE, 2, 1 },
-            { TRUE, 2, 1 },
-            { TRUE, 2, 1 },
-            { TRUE, 0, 10 },
-            { TRUE, 100, 0 },
-            { TRUE, 3000, 4000 },
-            { TRUE, 100000, 100000 }
+            { false, 10003, 10003 },
+            { true, 2, 1 },
+            { true, 2, 1 },
+            { true, 2, 1 },
+            { true, 0, 10 },
+            { true, 100, 0 },
+            { true, 3000, 4000 },
+            { true, 100000, 100000 }
     };
     TestUtility::checkEditsIter(*this, u"fine",
             edits.getFineIterator(), edits.getFineIterator(),
-            fineExpectedChanges, UPRV_LENGTHOF(fineExpectedChanges), TRUE, errorCode);
+            fineExpectedChanges, UPRV_LENGTHOF(fineExpectedChanges), true, errorCode);
     TestUtility::checkEditsIter(*this, u"fine changes",
             edits.getFineChangesIterator(), edits.getFineChangesIterator(),
-            fineExpectedChanges, UPRV_LENGTHOF(fineExpectedChanges), FALSE, errorCode);
+            fineExpectedChanges, UPRV_LENGTHOF(fineExpectedChanges), false, errorCode);
 
     edits.reset();
     assertFalse("reset hasChanges", edits.hasChanges());
@@ -1008,7 +1133,7 @@ void StringCaseTest::TestCopyMoveEdits() {
     TestUtility::checkEqualEdits(*this, u"c = b", b, c, errorCode);
 
     // std::move trouble on these platforms.
-    // See https://ssl.icu-project.org/trac/ticket/13393
+    // See https://unicode-org.atlassian.net/browse/ICU-13393
 #if !(U_PLATFORM == U_PF_AIX || U_PLATFORM == U_PF_OS390)
     // move constructor empties object with heap array
     Edits d(std::move(a));
@@ -1229,39 +1354,39 @@ void StringCaseTest::TestMergeEdits() {
 
 void StringCaseTest::TestCaseMapWithEdits() {
     IcuTestErrorCode errorCode(*this, "TestCaseMapWithEdits");
-    UChar dest[20];
+    char16_t dest[20];
     Edits edits;
 
     int32_t length = CaseMap::toLower("tr", U_OMIT_UNCHANGED_TEXT,
                                       u"IstanBul", 8, dest, UPRV_LENGTHOF(dest), &edits, errorCode);
-    assertEquals(u"toLower(IstanBul)", UnicodeString(u"ıb"), UnicodeString(TRUE, dest, length));
+    assertEquals(u"toLower(IstanBul)", UnicodeString(u"ıb"), UnicodeString(true, dest, length));
     static const EditChange lowerExpectedChanges[] = {
-            { TRUE, 1, 1 },
-            { FALSE, 4, 4 },
-            { TRUE, 1, 1 },
-            { FALSE, 2, 2 }
+            { true, 1, 1 },
+            { false, 4, 4 },
+            { true, 1, 1 },
+            { false, 2, 2 }
     };
     TestUtility::checkEditsIter(*this, u"toLower(IstanBul)",
             edits.getFineIterator(), edits.getFineIterator(),
             lowerExpectedChanges, UPRV_LENGTHOF(lowerExpectedChanges),
-            TRUE, errorCode);
+            true, errorCode);
 
     edits.reset();
     length = CaseMap::toUpper("el", U_OMIT_UNCHANGED_TEXT,
                               u"Πατάτα", 6, dest, UPRV_LENGTHOF(dest), &edits, errorCode);
-    assertEquals(u"toUpper(Πατάτα)", UnicodeString(u"ΑΤΑΤΑ"), UnicodeString(TRUE, dest, length));
+    assertEquals(u"toUpper(Πατάτα)", UnicodeString(u"ΑΤΑΤΑ"), UnicodeString(true, dest, length));
     static const EditChange upperExpectedChanges[] = {
-            { FALSE, 1, 1 },
-            { TRUE, 1, 1 },
-            { TRUE, 1, 1 },
-            { TRUE, 1, 1 },
-            { TRUE, 1, 1 },
-            { TRUE, 1, 1 }
+            { false, 1, 1 },
+            { true, 1, 1 },
+            { true, 1, 1 },
+            { true, 1, 1 },
+            { true, 1, 1 },
+            { true, 1, 1 }
     };
     TestUtility::checkEditsIter(*this, u"toUpper(Πατάτα)",
             edits.getFineIterator(), edits.getFineIterator(),
             upperExpectedChanges, UPRV_LENGTHOF(upperExpectedChanges),
-            TRUE, errorCode);
+            true, errorCode);
 
     edits.reset();
 
@@ -1272,40 +1397,40 @@ void StringCaseTest::TestCaseMapWithEdits() {
                               U_TITLECASE_NO_LOWERCASE,
                               nullptr, u"IjssEL IglOo", 12,
                               dest, UPRV_LENGTHOF(dest), &edits, errorCode);
-    assertEquals(u"toTitle(IjssEL IglOo)", UnicodeString(u"J"), UnicodeString(TRUE, dest, length));
+    assertEquals(u"toTitle(IjssEL IglOo)", UnicodeString(u"J"), UnicodeString(true, dest, length));
     static const EditChange titleExpectedChanges[] = {
-            { FALSE, 1, 1 },
-            { TRUE, 1, 1 },
-            { FALSE, 10, 10 }
+            { false, 1, 1 },
+            { true, 1, 1 },
+            { false, 10, 10 }
     };
     TestUtility::checkEditsIter(*this, u"toTitle(IjssEL IglOo)",
             edits.getFineIterator(), edits.getFineIterator(),
             titleExpectedChanges, UPRV_LENGTHOF(titleExpectedChanges),
-            TRUE, errorCode);
+            true, errorCode);
 #endif
 
     // No explicit nor automatic edits.reset(). Edits should be appended.
     length = CaseMap::fold(U_OMIT_UNCHANGED_TEXT | U_EDITS_NO_RESET | U_FOLD_CASE_EXCLUDE_SPECIAL_I,
                            u"IßtanBul", 8, dest, UPRV_LENGTHOF(dest), &edits, errorCode);
-    assertEquals(u"foldCase(IßtanBul)", UnicodeString(u"ıssb"), UnicodeString(TRUE, dest, length));
+    assertEquals(u"foldCase(IßtanBul)", UnicodeString(u"ıssb"), UnicodeString(true, dest, length));
     static const EditChange foldExpectedChanges[] = {
 #if !UCONFIG_NO_BREAK_ITERATION
             // From titlecasing.
-            { FALSE, 1, 1 },
-            { TRUE, 1, 1 },
-            { FALSE, 10, 10 },
+            { false, 1, 1 },
+            { true, 1, 1 },
+            { false, 10, 10 },
 #endif
             // From case folding.
-            { TRUE, 1, 1 },
-            { TRUE, 1, 2 },
-            { FALSE, 3, 3 },
-            { TRUE, 1, 1 },
-            { FALSE, 2, 2 }
+            { true, 1, 1 },
+            { true, 1, 2 },
+            { false, 3, 3 },
+            { true, 1, 1 },
+            { false, 2, 2 }
     };
     TestUtility::checkEditsIter(*this, u"foldCase(no Edits reset, IßtanBul)",
             edits.getFineIterator(), edits.getFineIterator(),
             foldExpectedChanges, UPRV_LENGTHOF(foldExpectedChanges),
-            TRUE, errorCode);
+            true, errorCode);
 }
 
 void StringCaseTest::TestCaseMapUTF8WithEdits() {
@@ -1314,37 +1439,39 @@ void StringCaseTest::TestCaseMapUTF8WithEdits() {
     Edits edits;
 
     int32_t length = CaseMap::utf8ToLower("tr", U_OMIT_UNCHANGED_TEXT,
-                                          u8"IstanBul", 8, dest, UPRV_LENGTHOF(dest), &edits, errorCode);
+                                          reinterpret_cast<const char*>(u8"IstanBul"), 8,
+                                          dest, UPRV_LENGTHOF(dest), &edits, errorCode);
     assertEquals(u"toLower(IstanBul)", UnicodeString(u"ıb"),
                  UnicodeString::fromUTF8(StringPiece(dest, length)));
     static const EditChange lowerExpectedChanges[] = {
-            { TRUE, 1, 2 },
-            { FALSE, 4, 4 },
-            { TRUE, 1, 1 },
-            { FALSE, 2, 2 }
+            { true, 1, 2 },
+            { false, 4, 4 },
+            { true, 1, 1 },
+            { false, 2, 2 }
     };
     TestUtility::checkEditsIter(*this, u"toLower(IstanBul)",
             edits.getFineIterator(), edits.getFineIterator(),
             lowerExpectedChanges, UPRV_LENGTHOF(lowerExpectedChanges),
-            TRUE, errorCode);
+            true, errorCode);
 
     edits.reset();
     length = CaseMap::utf8ToUpper("el", U_OMIT_UNCHANGED_TEXT,
-                                  u8"Πατάτα", 6 * 2, dest, UPRV_LENGTHOF(dest), &edits, errorCode);
+                                  reinterpret_cast<const char*>(u8"Πατάτα"), 6 * 2,
+                                  dest, UPRV_LENGTHOF(dest), &edits, errorCode);
     assertEquals(u"toUpper(Πατάτα)", UnicodeString(u"ΑΤΑΤΑ"),
                  UnicodeString::fromUTF8(StringPiece(dest, length)));
     static const EditChange upperExpectedChanges[] = {
-            { FALSE, 2, 2 },
-            { TRUE, 2, 2 },
-            { TRUE, 2, 2 },
-            { TRUE, 2, 2 },
-            { TRUE, 2, 2 },
-            { TRUE, 2, 2 }
+            { false, 2, 2 },
+            { true, 2, 2 },
+            { true, 2, 2 },
+            { true, 2, 2 },
+            { true, 2, 2 },
+            { true, 2, 2 }
     };
     TestUtility::checkEditsIter(*this, u"toUpper(Πατάτα)",
             edits.getFineIterator(), edits.getFineIterator(),
             upperExpectedChanges, UPRV_LENGTHOF(upperExpectedChanges),
-            TRUE, errorCode);
+            true, errorCode);
 
     edits.reset();
 #if !UCONFIG_NO_BREAK_ITERATION
@@ -1352,45 +1479,46 @@ void StringCaseTest::TestCaseMapUTF8WithEdits() {
                                   U_OMIT_UNCHANGED_TEXT |
                                   U_TITLECASE_NO_BREAK_ADJUSTMENT |
                                   U_TITLECASE_NO_LOWERCASE,
-                                  nullptr, u8"IjssEL IglOo", 12,
+                                  nullptr, reinterpret_cast<const char*>(u8"IjssEL IglOo"), 12,
                                   dest, UPRV_LENGTHOF(dest), &edits, errorCode);
     assertEquals(u"toTitle(IjssEL IglOo)", UnicodeString(u"J"),
                  UnicodeString::fromUTF8(StringPiece(dest, length)));
     static const EditChange titleExpectedChanges[] = {
-            { FALSE, 1, 1 },
-            { TRUE, 1, 1 },
-            { FALSE, 10, 10 }
+            { false, 1, 1 },
+            { true, 1, 1 },
+            { false, 10, 10 }
     };
     TestUtility::checkEditsIter(*this, u"toTitle(IjssEL IglOo)",
             edits.getFineIterator(), edits.getFineIterator(),
             titleExpectedChanges, UPRV_LENGTHOF(titleExpectedChanges),
-            TRUE, errorCode);
+            true, errorCode);
 #endif
 
     // No explicit nor automatic edits.reset(). Edits should be appended.
     length = CaseMap::utf8Fold(U_OMIT_UNCHANGED_TEXT | U_EDITS_NO_RESET |
                                    U_FOLD_CASE_EXCLUDE_SPECIAL_I,
-                               u8"IßtanBul", 1 + 2 + 6, dest, UPRV_LENGTHOF(dest), &edits, errorCode);
+                               reinterpret_cast<const char*>(u8"IßtanBul"), 1 + 2 + 6,
+                               dest, UPRV_LENGTHOF(dest), &edits, errorCode);
     assertEquals(u"foldCase(IßtanBul)", UnicodeString(u"ıssb"),
                  UnicodeString::fromUTF8(StringPiece(dest, length)));
     static const EditChange foldExpectedChanges[] = {
 #if !UCONFIG_NO_BREAK_ITERATION
             // From titlecasing.
-            { FALSE, 1, 1 },
-            { TRUE, 1, 1 },
-            { FALSE, 10, 10 },
+            { false, 1, 1 },
+            { true, 1, 1 },
+            { false, 10, 10 },
 #endif
             // From case folding.
-            { TRUE, 1, 2 },
-            { TRUE, 2, 2 },
-            { FALSE, 3, 3 },
-            { TRUE, 1, 1 },
-            { FALSE, 2, 2 }
+            { true, 1, 2 },
+            { true, 2, 2 },
+            { false, 3, 3 },
+            { true, 1, 1 },
+            { false, 2, 2 }
     };
     TestUtility::checkEditsIter(*this, u"foldCase(IßtanBul)",
             edits.getFineIterator(), edits.getFineIterator(),
             foldExpectedChanges, UPRV_LENGTHOF(foldExpectedChanges),
-            TRUE, errorCode);
+            true, errorCode);
 }
 
 void StringCaseTest::TestCaseMapToString() {
@@ -1398,17 +1526,17 @@ void StringCaseTest::TestCaseMapToString() {
     // It is a bit of a misnomer until we have CaseMap API that writes to
     // a UnicodeString, at which point we should change this code here.
     IcuTestErrorCode errorCode(*this, "TestCaseMapToString");
-    UChar dest[20];
+    char16_t dest[20];
 
     // Omit unchanged text.
     int32_t length = CaseMap::toLower("tr", U_OMIT_UNCHANGED_TEXT,
                                       u"IstanBul", 8, dest, UPRV_LENGTHOF(dest), nullptr, errorCode);
     assertEquals(u"toLower(IstanBul)",
-                 UnicodeString(u"ıb"), UnicodeString(TRUE, dest, length));
+                 UnicodeString(u"ıb"), UnicodeString(true, dest, length));
     length = CaseMap::toUpper("el", U_OMIT_UNCHANGED_TEXT,
                               u"Πατάτα", 6, dest, UPRV_LENGTHOF(dest), nullptr, errorCode);
     assertEquals(u"toUpper(Πατάτα)",
-                 UnicodeString(u"ΑΤΑΤΑ"), UnicodeString(TRUE, dest, length));
+                 UnicodeString(u"ΑΤΑΤΑ"), UnicodeString(true, dest, length));
 #if !UCONFIG_NO_BREAK_ITERATION
     length = CaseMap::toTitle("nl",
                               U_OMIT_UNCHANGED_TEXT |
@@ -1417,22 +1545,22 @@ void StringCaseTest::TestCaseMapToString() {
                               nullptr, u"IjssEL IglOo", 12,
                               dest, UPRV_LENGTHOF(dest), nullptr, errorCode);
     assertEquals(u"toTitle(IjssEL IglOo)",
-                 UnicodeString(u"J"), UnicodeString(TRUE, dest, length));
+                 UnicodeString(u"J"), UnicodeString(true, dest, length));
 #endif
     length = CaseMap::fold(U_OMIT_UNCHANGED_TEXT | U_FOLD_CASE_EXCLUDE_SPECIAL_I,
                            u"IßtanBul", 8, dest, UPRV_LENGTHOF(dest), nullptr, errorCode);
     assertEquals(u"foldCase(IßtanBul)",
-                 UnicodeString(u"ıssb"), UnicodeString(TRUE, dest, length));
+                 UnicodeString(u"ıssb"), UnicodeString(true, dest, length));
 
     // Return the whole result string.
     length = CaseMap::toLower("tr", 0,
                               u"IstanBul", 8, dest, UPRV_LENGTHOF(dest), nullptr, errorCode);
     assertEquals(u"toLower(IstanBul)",
-                 UnicodeString(u"ıstanbul"), UnicodeString(TRUE, dest, length));
+                 UnicodeString(u"ıstanbul"), UnicodeString(true, dest, length));
     length = CaseMap::toUpper("el", 0,
                               u"Πατάτα", 6, dest, UPRV_LENGTHOF(dest), nullptr, errorCode);
     assertEquals(u"toUpper(Πατάτα)",
-                 UnicodeString(u"ΠΑΤΑΤΑ"), UnicodeString(TRUE, dest, length));
+                 UnicodeString(u"ΠΑΤΑΤΑ"), UnicodeString(true, dest, length));
 #if !UCONFIG_NO_BREAK_ITERATION
     length = CaseMap::toTitle("nl",
                               U_TITLECASE_NO_BREAK_ADJUSTMENT |
@@ -1440,12 +1568,12 @@ void StringCaseTest::TestCaseMapToString() {
                               nullptr, u"IjssEL IglOo", 12,
                               dest, UPRV_LENGTHOF(dest), nullptr, errorCode);
     assertEquals(u"toTitle(IjssEL IglOo)",
-                 UnicodeString(u"IJssEL IglOo"), UnicodeString(TRUE, dest, length));
+                 UnicodeString(u"IJssEL IglOo"), UnicodeString(true, dest, length));
 #endif
     length = CaseMap::fold(U_FOLD_CASE_EXCLUDE_SPECIAL_I,
                            u"IßtanBul", 8, dest, UPRV_LENGTHOF(dest), nullptr, errorCode);
     assertEquals(u"foldCase(IßtanBul)",
-                 UnicodeString(u"ısstanbul"), UnicodeString(TRUE, dest, length));
+                 UnicodeString(u"ısstanbul"), UnicodeString(true, dest, length));
 }
 
 void StringCaseTest::TestCaseMapUTF8ToString() {
@@ -1499,16 +1627,16 @@ void StringCaseTest::TestCaseMapUTF8ToString() {
 void StringCaseTest::TestLongUnicodeString() {
     // Code coverage for UnicodeString case mapping code handling
     // long strings or many changes in a string.
-    UnicodeString s(TRUE,
-        (const UChar *)
+    UnicodeString s(true,
+        (const char16_t *)
         u"aaaaaaaaaabbbbbbbbbbccccccccccddddddddddeeeeeeeeeeF"
         u"aaaaaaaaaabbbbbbbbbbccccccccccddddddddddeeeeeeeeeeF"
         u"aaaaaaaaaabbbbbbbbbbccccccccccddddddddddeeeeeeeeeeF"
         u"aaaaaaaaaabbbbbbbbbbccccccccccddddddddddeeeeeeeeeeF"
         u"aaaaaaaaaabbbbbbbbbbccccccccccddddddddddeeeeeeeeeeF"
         u"aaaaaaaaaabbbbbbbbbbccccccccccddddddddddeeeeeeeeeeF", 6 * 51);
-    UnicodeString expected(TRUE,
-        (const UChar *)
+    UnicodeString expected(true,
+        (const char16_t *)
         u"AAAAAAAAAABBBBBBBBBBCCCCCCCCCCDDDDDDDDDDEEEEEEEEEEF"
         u"AAAAAAAAAABBBBBBBBBBCCCCCCCCCCDDDDDDDDDDEEEEEEEEEEF"
         u"AAAAAAAAAABBBBBBBBBBCCCCCCCCCCDDDDDDDDDDEEEEEEEEEEF"
@@ -1523,7 +1651,7 @@ void StringCaseTest::TestLongUnicodeString() {
 void StringCaseTest::TestBug13127() {
     // Test case crashed when the bug was present.
     const char16_t *s16 = u"日本語";
-    UnicodeString s(TRUE, s16, -1);
+    UnicodeString s(true, s16, -1);
     s.toTitle(0, Locale::getEnglish());
 }
 
