@@ -39,7 +39,7 @@ UnicodeTest::UnicodeTest()
     unknownPropertyNames=new U_NAMESPACE_QUALIFIER Hashtable(errorCode);
     if(U_FAILURE(errorCode)) {
         delete unknownPropertyNames;
-        unknownPropertyNames=NULL;
+        unknownPropertyNames=nullptr;
     }
     // Ignore some property names altogether.
     for(int32_t i=0; i<UPRV_LENGTHOF(ignorePropNames); ++i) {
@@ -65,6 +65,7 @@ void UnicodeTest::runIndexedTest( int32_t index, UBool exec, const char* &name,
     TESTCASE_AUTO(TestScriptMetadata);
     TESTCASE_AUTO(TestBidiPairedBracketType);
     TESTCASE_AUTO(TestEmojiProperties);
+    TESTCASE_AUTO(TestEmojiPropertiesOfStrings);
     TESTCASE_AUTO(TestIndicPositionalCategory);
     TESTCASE_AUTO(TestIndicSyllabicCategory);
     TESTCASE_AUTO(TestVerticalOrientation);
@@ -74,6 +75,7 @@ void UnicodeTest::runIndexedTest( int32_t index, UBool exec, const char* &name,
     TESTCASE_AUTO(TestBinaryCharacterProperties);
     TESTCASE_AUTO(TestIntCharacterProperties);
 #endif
+    TESTCASE_AUTO(TestPropertyNames);
     TESTCASE_AUTO_END;
 }
 
@@ -92,7 +94,7 @@ getTokenIndex(const char *const tokens[], int32_t countTokens, const char *s) {
     s=u_skipWhitespace(s);
     for(i=0; i<countTokens; ++i) {
         t=tokens[i];
-        if(t!=NULL) {
+        if(t!=nullptr) {
             for(j=0;; ++j) {
                 if(t[j]!=0) {
                     if(s[j]!=t[j]) {
@@ -171,7 +173,7 @@ derivedPropsLineFn(void *context,
                    char *fields[][2], int32_t /* fieldCount */,
                    UErrorCode *pErrorCode)
 {
-    UnicodeTest *me=(UnicodeTest *)context;
+    UnicodeTest *me=static_cast<UnicodeTest*>(context);
     uint32_t start, end;
     int32_t i;
 
@@ -186,7 +188,7 @@ derivedPropsLineFn(void *context,
     if(i<0) {
         UnicodeString propName(fields[1][0], (int32_t)(fields[1][1]-fields[1][0]));
         propName.trim();
-        if(me->unknownPropertyNames->find(propName)==NULL) {
+        if(me->unknownPropertyNames->find(propName)==nullptr) {
             UErrorCode errorCode=U_ZERO_ERROR;
             me->unknownPropertyNames->puti(propName, 1, errorCode);
             me->errln("UnicodeTest warning: unknown property name '%s' in DerivedCoreProperties.txt or DerivedNormalizationProps.txt\n", fields[1][0]);
@@ -211,7 +213,7 @@ void UnicodeTest::TestAdditionalProperties() {
     }
 
     char path[500];
-    if(getUnidataPath(path) == NULL) {
+    if(getUnidataPath(path) == nullptr) {
         errln("unable to find path to source/data/unidata/");
         return;
     }
@@ -239,7 +241,7 @@ void UnicodeTest::TestAdditionalProperties() {
     uint32_t i;
     UChar32 start, end;
 
-    // test all TRUE properties
+    // test all true properties
     for(i=0; i<UPRV_LENGTHOF(derivedPropsNames); ++i) {
         rangeCount=derivedProps[i].getRangeCount();
         for(range=0; range<rangeCount && numErrors[i]<MAX_ERRORS; ++range) {
@@ -247,7 +249,7 @@ void UnicodeTest::TestAdditionalProperties() {
             end=derivedProps[i].getRangeEnd(range);
             for(; start<=end; ++start) {
                 if(!u_hasBinaryProperty(start, derivedPropsIndex[i])) {
-                    dataerrln("UnicodeTest error: u_hasBinaryProperty(U+%04lx, %s)==FALSE is wrong", start, derivedPropsNames[i]);
+                    dataerrln("UnicodeTest error: u_hasBinaryProperty(U+%04lx, %s)==false is wrong", start, derivedPropsNames[i]);
                     if(++numErrors[i]>=MAX_ERRORS) {
                       dataerrln("Too many errors, moving to the next test");
                       break;
@@ -262,7 +264,7 @@ void UnicodeTest::TestAdditionalProperties() {
         derivedProps[i].complement();
     }
 
-    // test all FALSE properties
+    // test all false properties
     for(i=0; i<UPRV_LENGTHOF(derivedPropsNames); ++i) {
         rangeCount=derivedProps[i].getRangeCount();
         for(range=0; range<rangeCount && numErrors[i]<MAX_ERRORS; ++range) {
@@ -270,7 +272,7 @@ void UnicodeTest::TestAdditionalProperties() {
             end=derivedProps[i].getRangeEnd(range);
             for(; start<=end; ++start) {
                 if(u_hasBinaryProperty(start, derivedPropsIndex[i])) {
-                    errln("UnicodeTest error: u_hasBinaryProperty(U+%04lx, %s)==TRUE is wrong\n", start, derivedPropsNames[i]);
+                    errln("UnicodeTest error: u_hasBinaryProperty(U+%04lx, %s)==true is wrong\n", start, derivedPropsNames[i]);
                     if(++numErrors[i]>=MAX_ERRORS) {
                       errln("Too many errors, moving to the next test");
                       break;
@@ -350,7 +352,7 @@ void UnicodeTest::TestConsistency() {
     UnicodeSet set1, set2;
     if (nfcImpl->getCanonStartSet(0x49, set1)) {
         /* enumerate all characters that are plausible to be latin letters */
-        for(UChar start=0xa0; start<0x2000; ++start) {
+        for(char16_t start=0xa0; start<0x2000; ++start) {
             UnicodeString decomp=nfd->normalize(UnicodeString(start), errorCode);
             if(decomp.length()>1 && decomp[0]==0x49) {
                 set2.add(start);
@@ -364,9 +366,9 @@ void UnicodeTest::TestConsistency() {
         // because the new internal normalization functions are in C++.
         //compareUSets(set1, set2,
         //             "[canon start set of 0049]", "[all c with canon decomp with 0049]",
-        //             TRUE);
+        //             true);
     } else {
-        errln("NFC.getCanonStartSet() returned FALSE");
+        errln("NFC.getCanonStartSet() returned false");
     }
 #endif
 }
@@ -401,16 +403,16 @@ void UnicodeTest::TestPatternProperties() {
         }
     }
     compareUSets(syn_pp, syn_prop,
-                 "PatternProps.isSyntax()", "[:Pattern_Syntax:]", TRUE);
+                 "PatternProps.isSyntax()", "[:Pattern_Syntax:]", true);
     compareUSets(syn_pp, syn_list,
-                 "PatternProps.isSyntax()", "[Pattern_Syntax ranges]", TRUE);
+                 "PatternProps.isSyntax()", "[Pattern_Syntax ranges]", true);
     compareUSets(ws_pp, ws_prop,
-                 "PatternProps.isWhiteSpace()", "[:Pattern_White_Space:]", TRUE);
+                 "PatternProps.isWhiteSpace()", "[:Pattern_White_Space:]", true);
     compareUSets(ws_pp, ws_list,
-                 "PatternProps.isWhiteSpace()", "[Pattern_White_Space ranges]", TRUE);
+                 "PatternProps.isWhiteSpace()", "[Pattern_White_Space ranges]", true);
     compareUSets(syn_ws_pp, syn_ws_prop,
                  "PatternProps.isSyntaxOrWhiteSpace()",
-                 "[[:Pattern_Syntax:][:Pattern_White_Space:]]", TRUE);
+                 "[[:Pattern_Syntax:][:Pattern_White_Space:]]", true);
 }
 
 // So far only minimal port of Java & cucdtst.c compareUSets().
@@ -545,6 +547,177 @@ void UnicodeTest::TestEmojiProperties() {
                u_hasBinaryProperty(0xA9, UCHAR_EXTENDED_PICTOGRAPHIC));
 }
 
+namespace {
+
+UBool hbp(const char16_t *s, int32_t length, UProperty which) {
+    return u_stringHasBinaryProperty(s, length, which);
+}
+
+UBool hbp(const char16_t *s, UProperty which) {
+    return u_stringHasBinaryProperty(s, -1, which);
+}
+
+}  // namespace
+
+void UnicodeTest::TestEmojiPropertiesOfStrings() {
+    // Property of code points, for coverage
+    assertFalse("null is not Ideographic", hbp(nullptr, 1, UCHAR_IDEOGRAPHIC));
+    assertFalse("null/0 is not Ideographic", hbp(nullptr, -1, UCHAR_IDEOGRAPHIC));
+    assertFalse("empty string is not Ideographic", hbp(u"", 0, UCHAR_IDEOGRAPHIC));
+    assertFalse("empty string/0 is not Ideographic", hbp(u"", -1, UCHAR_IDEOGRAPHIC));
+    assertFalse("L is not Ideographic", hbp(u"L", 1, UCHAR_IDEOGRAPHIC));
+    assertFalse("L/0 is not Ideographic", hbp(u"L", -1, UCHAR_IDEOGRAPHIC));
+    assertTrue("U+4E02 is Ideographic", hbp(u"丂", 1, UCHAR_IDEOGRAPHIC));
+    assertTrue("U+4E02/0 is Ideographic", hbp(u"丂", -1, UCHAR_IDEOGRAPHIC));
+    assertFalse("2*U+4E02 is not Ideographic", hbp(u"丂丂", 2, UCHAR_IDEOGRAPHIC));
+    assertFalse("2*U+4E02/0 is not Ideographic", hbp(u"丂丂", -1, UCHAR_IDEOGRAPHIC));
+    assertFalse("bicycle is not Ideographic", hbp(u"🚲", 2, UCHAR_IDEOGRAPHIC));
+    assertFalse("bicycle/0 is not Ideographic", hbp(u"🚲", -1, UCHAR_IDEOGRAPHIC));
+    assertTrue("U+23456 is Ideographic", hbp(u"\U00023456", 2, UCHAR_IDEOGRAPHIC));
+    assertTrue("U+23456/0 is Ideographic", hbp(u"\U00023456", -1, UCHAR_IDEOGRAPHIC));
+
+    // Property of (code points and) strings
+    assertFalse("null is not Basic_Emoji", hbp(nullptr, 1, UCHAR_BASIC_EMOJI));
+    assertFalse("null/0 is not Basic_Emoji", hbp(nullptr, -1, UCHAR_BASIC_EMOJI));
+    assertFalse("empty string is not Basic_Emoji", hbp(u"", 0, UCHAR_BASIC_EMOJI));
+    assertFalse("empty string/0 is not Basic_Emoji", hbp(u"", -1, UCHAR_BASIC_EMOJI));
+    assertFalse("L is not Basic_Emoji", hbp(u"L", 1, UCHAR_BASIC_EMOJI));
+    assertFalse("L/0 is not Basic_Emoji", hbp(u"L", -1, UCHAR_BASIC_EMOJI));
+    assertFalse("U+4E02 is not Basic_Emoji", hbp(u"丂", 1, UCHAR_BASIC_EMOJI));
+    assertFalse("U+4E02/0 is not Basic_Emoji", hbp(u"丂", -1, UCHAR_BASIC_EMOJI));
+    assertTrue("bicycle is Basic_Emoji", hbp(u"🚲", 2, UCHAR_BASIC_EMOJI));
+    assertTrue("bicycle/0 is Basic_Emoji", hbp(u"🚲", -1, UCHAR_BASIC_EMOJI));
+    assertFalse("2*bicycle is Basic_Emoji", hbp(u"🚲🚲", 4, UCHAR_BASIC_EMOJI));
+    assertFalse("2*bicycle/0 is Basic_Emoji", hbp(u"🚲🚲", -1, UCHAR_BASIC_EMOJI));
+    assertFalse("U+23456 is not Basic_Emoji", hbp(u"\U00023456", 2, UCHAR_BASIC_EMOJI));
+    assertFalse("U+23456/0 is not Basic_Emoji", hbp(u"\U00023456", -1, UCHAR_BASIC_EMOJI));
+
+    assertFalse("stopwatch is not Basic_Emoji", hbp(u"⏱", 1, UCHAR_BASIC_EMOJI));
+    assertFalse("stopwatch/0 is not Basic_Emoji", hbp(u"⏱", -1, UCHAR_BASIC_EMOJI));
+    assertTrue("stopwatch+emoji is Basic_Emoji", hbp(u"⏱\uFE0F", 2, UCHAR_BASIC_EMOJI));
+    assertTrue("stopwatch+emoji/0 is Basic_Emoji", hbp(u"⏱\uFE0F", -1, UCHAR_BASIC_EMOJI));
+
+    assertFalse("chipmunk is not Basic_Emoji", hbp(u"🐿", UCHAR_BASIC_EMOJI));
+    assertTrue("chipmunk+emoji is Basic_Emoji", hbp(u"🐿\uFE0F", UCHAR_BASIC_EMOJI));
+    assertFalse("chipmunk+2*emoji is not Basic_Emoji", hbp(u"🐿\uFE0F\uFE0F", UCHAR_BASIC_EMOJI));
+
+    // Properties of strings (only)
+    assertFalse("4+emoji is not Emoji_Keycap_Sequence",
+                hbp(u"4\uFE0F", UCHAR_EMOJI_KEYCAP_SEQUENCE));
+    assertTrue("4+emoji+keycap is Emoji_Keycap_Sequence",
+               hbp(u"4\uFE0F\u20E3", UCHAR_EMOJI_KEYCAP_SEQUENCE));
+
+    assertFalse("[B] is not RGI_Emoji_Flag_Sequence",
+                hbp(u"\U0001F1E7", UCHAR_RGI_EMOJI_FLAG_SEQUENCE));
+    assertTrue("[BE] is RGI_Emoji_Flag_Sequence",
+               hbp(u"🇧🇪", UCHAR_RGI_EMOJI_FLAG_SEQUENCE));
+
+    assertFalse("[flag] is not RGI_Emoji_Tag_Sequence",
+                hbp(u"\U0001F3F4", UCHAR_RGI_EMOJI_TAG_SEQUENCE));
+    assertTrue("[Scotland] is RGI_Emoji_Tag_Sequence",
+               hbp(u"🏴󠁧󠁢󠁳󠁣󠁴󠁿", UCHAR_RGI_EMOJI_TAG_SEQUENCE));
+
+    assertFalse("bicyclist is not RGI_Emoji_Modifier_Sequence",
+                hbp(u"🚴", UCHAR_RGI_EMOJI_MODIFIER_SEQUENCE));
+    assertTrue("bicyclist+medium is RGI_Emoji_Modifier_Sequence",
+               hbp(u"🚴\U0001F3FD", UCHAR_RGI_EMOJI_MODIFIER_SEQUENCE));
+
+    assertFalse("woman+dark+ZWJ is not RGI_Emoji_ZWJ_Sequence",
+                hbp(u"👩\U0001F3FF\u200D", UCHAR_RGI_EMOJI_ZWJ_SEQUENCE));
+    assertTrue("woman pilot: dark skin tone is RGI_Emoji_ZWJ_Sequence",
+               hbp(u"👩\U0001F3FF\u200D✈\uFE0F", UCHAR_RGI_EMOJI_ZWJ_SEQUENCE));
+
+    // RGI_Emoji = all of the above
+    assertFalse("stopwatch is not RGI_Emoji", hbp(u"⏱", UCHAR_RGI_EMOJI));
+    assertTrue("stopwatch+emoji is RGI_Emoji", hbp(u"⏱\uFE0F", UCHAR_RGI_EMOJI));
+
+    assertFalse("chipmunk is not RGI_Emoji", hbp(u"🐿", UCHAR_RGI_EMOJI));
+    assertTrue("chipmunk+emoji is RGI_Emoji", hbp(u"🐿\uFE0F", UCHAR_RGI_EMOJI));
+
+    assertFalse("4+emoji is not RGI_Emoji", hbp(u"4\uFE0F", UCHAR_RGI_EMOJI));
+    assertTrue("4+emoji+keycap is RGI_Emoji", hbp(u"4\uFE0F\u20E3", UCHAR_RGI_EMOJI));
+
+    assertFalse("[B] is not RGI_Emoji", hbp(u"\U0001F1E7", UCHAR_RGI_EMOJI));
+    assertTrue("[BE] is RGI_Emoji", hbp(u"🇧🇪", UCHAR_RGI_EMOJI));
+
+    assertTrue("[flag] is RGI_Emoji", hbp(u"\U0001F3F4", UCHAR_RGI_EMOJI));
+    assertTrue("[Scotland] is RGI_Emoji", hbp(u"🏴󠁧󠁢󠁳󠁣󠁴󠁿", UCHAR_RGI_EMOJI));
+
+    assertTrue("bicyclist is RGI_Emoji", hbp(u"🚴", UCHAR_RGI_EMOJI));
+    assertTrue("bicyclist+medium is RGI_Emoji", hbp(u"🚴\U0001F3FD", UCHAR_RGI_EMOJI));
+
+    assertFalse("woman+dark+ZWJ is not RGI_Emoji", hbp(u"👩\U0001F3FF\u200D", UCHAR_RGI_EMOJI));
+    assertTrue("woman pilot: dark skin tone is RGI_Emoji",
+               hbp(u"👩\U0001F3FF\u200D✈\uFE0F", UCHAR_RGI_EMOJI));
+
+    // UnicodeSet with properties of strings
+    IcuTestErrorCode errorCode(*this, "TestEmojiPropertiesOfStrings()");
+    UnicodeSet basic("[:Basic_Emoji:]", errorCode);
+    UnicodeSet keycaps("[:Emoji_Keycap_Sequence:]", errorCode);
+    UnicodeSet modified("[:RGI_Emoji_Modifier_Sequence:]", errorCode);
+    UnicodeSet flags("[:RGI_Emoji_Flag_Sequence:]", errorCode);
+    UnicodeSet tags("[:RGI_Emoji_Tag_Sequence:]", errorCode);
+    UnicodeSet combos("[:RGI_Emoji_ZWJ_Sequence:]", errorCode);
+    UnicodeSet rgi("[:RGI_Emoji:]", errorCode);
+    if (errorCode.errDataIfFailureAndReset("UnicodeSets")) {
+        return;
+    }
+
+    // union of all sets except for "rgi" -- should be the same as "rgi"
+    UnicodeSet all(basic);
+    all.addAll(keycaps).addAll(modified).addAll(flags).addAll(tags).addAll(combos);
+
+    UnicodeSet basicOnlyCp(basic);
+    basicOnlyCp.removeAllStrings();
+
+    UnicodeSet rgiOnlyCp(rgi);
+    rgiOnlyCp.removeAllStrings();
+
+    assertTrue("lots of Basic_Emoji", basic.size() > 1000);
+    assertEquals("12 Emoji_Keycap_Sequence", 12, keycaps.size());
+    assertTrue("lots of RGI_Emoji_Modifier_Sequence", modified.size() > 600);
+    assertTrue("lots of RGI_Emoji_Flag_Sequence", flags.size() > 250);
+    assertTrue("some RGI_Emoji_Tag_Sequence", tags.size() >= 3);
+    assertTrue("lots of RGI_Emoji_ZWJ_Sequence", combos.size() > 1300);
+    assertTrue("lots of RGI_Emoji", rgi.size() > 3000);
+
+    assertTrue("lots of Basic_Emoji code points", basicOnlyCp.size() > 1000);
+    assertTrue("Basic_Emoji.hasStrings()", basic.hasStrings());
+    assertEquals("no Emoji_Keycap_Sequence code points", 0, keycaps.getRangeCount());
+    assertEquals("lots of RGI_Emoji_Modifier_Sequence", 0, modified.getRangeCount());
+    assertEquals("lots of RGI_Emoji_Flag_Sequence", 0, flags.getRangeCount());
+    assertEquals("some RGI_Emoji_Tag_Sequence", 0, tags.getRangeCount());
+    assertEquals("lots of RGI_Emoji_ZWJ_Sequence", 0, combos.getRangeCount());
+
+    assertTrue("lots of RGI_Emoji code points", rgiOnlyCp.size() > 1000);
+    assertTrue("RGI_Emoji.hasStrings()", rgi.hasStrings());
+    assertEquals("RGI_Emoji/only-cp.size() == Basic_Emoji/only-cp.size()",
+                 rgiOnlyCp.size(), basicOnlyCp.size());
+    assertTrue("RGI_Emoji/only-cp == Basic_Emoji/only-cp", rgiOnlyCp == basicOnlyCp);
+    assertEquals("RGI_Emoji.size() == union.size()", rgi.size(), all.size());
+    assertTrue("RGI_Emoji == union", rgi == all);
+
+    assertTrue("Basic_Emoji.contains(stopwatch+emoji)", basic.contains(u"⏱\uFE0F"));
+    assertTrue("Basic_Emoji.contains(chipmunk+emoji)", basic.contains(u"🐿\uFE0F"));
+    assertTrue("Emoji_Keycap_Sequence.contains(4+emoji+keycap)",
+               keycaps.contains(u"4\uFE0F\u20E3"));
+    assertTrue("RGI_Emoji_Flag_Sequence.contains([BE])", flags.contains(u"🇧🇪"));
+    assertTrue("RGI_Emoji_Tag_Sequence.contains([Scotland])", tags.contains(u"🏴󠁧󠁢󠁳󠁣󠁴󠁿"));
+    assertTrue("RGI_Emoji_Modifier_Sequence.contains(bicyclist+medium)",
+               modified.contains(u"🚴\U0001F3FD"));
+    assertTrue("RGI_Emoji_ZWJ_Sequence.contains(woman pilot: dark skin tone)",
+               combos.contains(u"👩\U0001F3FF\u200D✈\uFE0F"));
+    assertTrue("RGI_Emoji.contains(stopwatch+emoji)", rgi.contains(u"⏱\uFE0F"));
+    assertTrue("RGI_Emoji.contains(chipmunk+emoji)", rgi.contains(u"🐿\uFE0F"));
+    assertTrue("RGI_Emoji.contains(4+emoji+keycap)", rgi.contains(u"4\uFE0F\u20E3"));
+    assertTrue("RGI_Emoji.contains([BE] is RGI_Emoji)", rgi.contains(u"🇧🇪"));
+    assertTrue("RGI_Emoji.contains([flag])", rgi.contains(u"\U0001F3F4"));
+    assertTrue("RGI_Emoji.contains([Scotland])", rgi.contains(u"🏴󠁧󠁢󠁳󠁣󠁴󠁿"));
+    assertTrue("RGI_Emoji.contains(bicyclist)", rgi.contains(u"🚴"));
+    assertTrue("RGI_Emoji.contains(bicyclist+medium)", rgi.contains(u"🚴\U0001F3FD"));
+    assertTrue("RGI_Emoji.contains(woman pilot: dark skin tone)", rgi.contains(u"👩\U0001F3FF\u200D✈\uFE0F"));
+}
+
 void UnicodeTest::TestIndicPositionalCategory() {
     IcuTestErrorCode errorCode(*this, "TestIndicPositionalCategory()");
     UnicodeSet na(u"[:InPC=NA:]", errorCode);
@@ -604,7 +777,7 @@ void UnicodeTest::TestDefaultScriptExtensions() {
     assertEquals("U+3012 num scx[0]", USCRIPT_COMMON, scx[0]);
 }
 
-void UnicodeTest::TestInvalidCodePointFolding(void) {
+void UnicodeTest::TestInvalidCodePointFolding() {
     // Test behavior when an invalid code point is passed to u_foldCase
     static const UChar32 invalidCodePoints[] = {
             0xD800, // lead surrogate
@@ -633,8 +806,8 @@ void UnicodeTest::TestBinaryCharacterProperties() {
             continue;
         }
         const UnicodeSet &set = *UnicodeSet::fromUSet(uset);
-        int32_t size = set.size();
-        if (size == 0) {
+        int32_t count = set.getRangeCount();
+        if (count == 0) {
             assertFalse(UnicodeString("!hasBinaryProperty(U+0020, ") + prop + u")",
                 u_hasBinaryProperty(0x20, (UProperty)prop));
             assertFalse(UnicodeString("!hasBinaryProperty(U+0061, ") + prop + u")",
@@ -642,7 +815,7 @@ void UnicodeTest::TestBinaryCharacterProperties() {
             assertFalse(UnicodeString("!hasBinaryProperty(U+4E00, ") + prop + u")",
                 u_hasBinaryProperty(0x4e00, (UProperty)prop));
         } else {
-            UChar32 c = set.charAt(0);
+            UChar32 c = set.getRangeStart(0);
             if (c > 0) {
                 assertFalse(
                     UnicodeString("!hasBinaryProperty(") + TestUtility::hex(c - 1) +
@@ -653,7 +826,7 @@ void UnicodeTest::TestBinaryCharacterProperties() {
                 UnicodeString("hasBinaryProperty(") + TestUtility::hex(c) +
                     u", " + prop + u")",
                 u_hasBinaryProperty(c, (UProperty)prop));
-            c = set.charAt(size - 1);
+            c = set.getRangeEnd(count - 1);
             assertTrue(
                 UnicodeString("hasBinaryProperty(") + TestUtility::hex(c) +
                     u", " + prop + u")",
@@ -696,3 +869,82 @@ void UnicodeTest::TestIntCharacterProperties() {
     }
 #endif
 }
+
+namespace {
+
+const char *getPropName(UProperty property, int32_t nameChoice) UPRV_NO_SANITIZE_UNDEFINED {
+    const char *name = u_getPropertyName(property, (UPropertyNameChoice)nameChoice);
+    return name != nullptr ? name : "null";
+}
+
+const char *getValueName(UProperty property, int32_t value, int32_t nameChoice)
+        UPRV_NO_SANITIZE_UNDEFINED {
+    const char *name = u_getPropertyValueName(property, value, (UPropertyNameChoice)nameChoice);
+    return name != nullptr ? name : "null";
+}
+
+}  // namespace
+
+void UnicodeTest::TestPropertyNames() {
+    IcuTestErrorCode errorCode(*this, "TestPropertyNames()");
+    // Test names of certain properties & values.
+    // The UPropertyNameChoice is really an integer with only a couple of named constants.
+    UProperty prop = UCHAR_WHITE_SPACE;
+    constexpr int32_t SHORT = U_SHORT_PROPERTY_NAME;
+    constexpr int32_t LONG = U_LONG_PROPERTY_NAME;
+    assertEquals("White_Space: index -1", "null", getPropName(prop, -1));
+    assertEquals("White_Space: short", "WSpace", getPropName(prop, SHORT));
+    assertEquals("White_Space: long", "White_Space", getPropName(prop, LONG));
+    assertEquals("White_Space: index 2", "space", getPropName(prop, 2));
+    assertEquals("White_Space: index 3", "null", getPropName(prop, 3));
+
+    prop = UCHAR_SIMPLE_CASE_FOLDING;
+    assertEquals("Simple_Case_Folding: index -1", "null", getPropName(prop, -1));
+    assertEquals("Simple_Case_Folding: short", "scf", getPropName(prop, SHORT));
+    assertEquals("Simple_Case_Folding: long", "Simple_Case_Folding", getPropName(prop, LONG));
+    assertEquals("Simple_Case_Folding: index 2", "sfc", getPropName(prop, 2));
+    assertEquals("Simple_Case_Folding: index 3", "null", getPropName(prop, 3));
+
+    prop = UCHAR_CASED;
+    assertEquals("Cased=Y: index -1", "null", getValueName(prop, 1, -1));
+    assertEquals("Cased=Y: short", "Y", getValueName(prop, 1, SHORT));
+    assertEquals("Cased=Y: long", "Yes", getValueName(prop, 1, LONG));
+    assertEquals("Cased=Y: index 2", "T", getValueName(prop, 1, 2));
+    assertEquals("Cased=Y: index 3", "True", getValueName(prop, 1, 3));
+    assertEquals("Cased=Y: index 4", "null", getValueName(prop, 1, 4));
+
+    prop = UCHAR_DECOMPOSITION_TYPE;
+    int32_t value = U_DT_NOBREAK;
+    assertEquals("dt=Nb: index -1", "null", getValueName(prop, value, -1));
+    assertEquals("dt=Nb: short", "Nb", getValueName(prop, value, SHORT));
+    assertEquals("dt=Nb: long", "Nobreak", getValueName(prop, value, LONG));
+    assertEquals("dt=Nb: index 2", "nb", getValueName(prop, value, 2));
+    assertEquals("dt=Nb: index 3", "null", getValueName(prop, value, 3));
+
+    // Canonical_Combining_Class:
+    // The UCD inserts the numeric values in the second filed of its PropertyValueAliases.txt lines.
+    // In ICU, we don't treat these as names,
+    // they are just the numeric values returned by u_getCombiningClass().
+    // We return the real short and long names for the usual choice constants.
+    prop = UCHAR_CANONICAL_COMBINING_CLASS;
+    assertEquals("ccc=230: index -1", "null", getValueName(prop, 230, -1));
+    assertEquals("ccc=230: short", "A", getValueName(prop, 230, SHORT));
+    assertEquals("ccc=230: long", "Above", getValueName(prop, 230, LONG));
+    assertEquals("ccc=230: index 2", "null", getValueName(prop, 230, 2));
+
+    prop = UCHAR_GENERAL_CATEGORY;
+    value = U_DECIMAL_DIGIT_NUMBER;
+    assertEquals("gc=Nd: index -1", "null", getValueName(prop, value, -1));
+    assertEquals("gc=Nd: short", "Nd", getValueName(prop, value, SHORT));
+    assertEquals("gc=Nd: long", "Decimal_Number", getValueName(prop, value, LONG));
+    assertEquals("gc=Nd: index 2", "digit", getValueName(prop, value, 2));
+    assertEquals("gc=Nd: index 3", "null", getValueName(prop, value, 3));
+
+    prop = UCHAR_GENERAL_CATEGORY_MASK;
+    value = U_GC_P_MASK;
+    assertEquals("gc=P mask: index -1", "null", getValueName(prop, value, -1));
+    assertEquals("gc=P mask: short", "P", getValueName(prop, value, SHORT));
+    assertEquals("gc=P mask: long", "Punctuation", getValueName(prop, value, LONG));
+    assertEquals("gc=P mask: index 2", "punct", getValueName(prop, value, 2));
+    assertEquals("gc=P mask: index 3", "null", getValueName(prop, value, 3));
+}
